

/* Registers declarations for the STM32F10x devices */

/* The addresses are fixed in the linker script */

/*------------------------ Analog to Digital Converter -----------------------*/
typedef struct
{
  volatile unsigned long SR;
  volatile unsigned long CR1;
  volatile unsigned long CR2;
  volatile unsigned long SMPR1;
  volatile unsigned long SMPR2;
  volatile unsigned long JOFR1;
  volatile unsigned long JOFR2;
  volatile unsigned long JOFR3;
  volatile unsigned long JOFR4;
  volatile unsigned long HTR;
  volatile unsigned long LTR;
  volatile unsigned long SQR1;
  volatile unsigned long SQR2;
  volatile unsigned long SQR3;
  volatile unsigned long JSQR;
  volatile unsigned long JDR1;
  volatile unsigned long JDR2;
  volatile unsigned long JDR3;
  volatile unsigned long JDR4;
  volatile unsigned long DR;
} ADC_TypeDef;

/*------------------------ Backup Registers ----------------------------------*/
typedef struct
{
  unsigned long RESERVED0;
  volatile unsigned short DR1;
  unsigned short  RESERVED1;
  volatile unsigned short DR2;
  unsigned short  RESERVED2;
  volatile unsigned short DR3;
  unsigned short  RESERVED3;
  volatile unsigned short DR4;
  unsigned short  RESERVED4;
  volatile unsigned short DR5;
  unsigned short  RESERVED5;
  volatile unsigned short DR6;
  unsigned short  RESERVED6;
  volatile unsigned short DR7;
  unsigned short  RESERVED7;
  volatile unsigned short DR8;
  unsigned short  RESERVED8;
  volatile unsigned short DR9;
  unsigned short  RESERVED9;
  volatile unsigned short DR10;
  unsigned short  RESERVED10;
  volatile unsigned short RTCCR;
  unsigned short  RESERVED11;
  volatile unsigned short CR;
  unsigned short  RESERVED12;
  volatile unsigned short CSR;
  unsigned short  RESERVED13;
} BKP_TypeDef;

/*------------------------ Controller Area Network ---------------------------*/
typedef struct
{
  volatile unsigned long TIR;
  volatile unsigned long TDTR;
  volatile unsigned long TDLR;
  volatile unsigned long TDHR;
} CAN_TxMailBox_TypeDef;

typedef struct
{
  volatile unsigned long RIR;
  volatile unsigned long RDTR;
  volatile unsigned long RDLR;
  volatile unsigned long RDHR;
} CAN_FIFOMailBox_TypeDef;

typedef struct
{
  volatile unsigned long FR0;
  volatile unsigned long FR1;
} CAN_FilterRegister_TypeDef;

typedef struct
{
  volatile unsigned long MCR;
  volatile unsigned long MSR;
  volatile unsigned long TSR;
  volatile unsigned long RF0R;
  volatile unsigned long RF1R;
  volatile unsigned long IER;
  volatile unsigned long ESR;
  volatile unsigned long BTR;
  unsigned long RESERVED0[88];
  CAN_TxMailBox_TypeDef sTxMailBox[3];
  CAN_FIFOMailBox_TypeDef sFIFOMailBox[2];
  unsigned long RESERVED1[12];
  volatile unsigned long FMR;
  volatile unsigned long FM0R;
  unsigned long RESERVED2[1];
  volatile unsigned long FS0R;
  unsigned long RESERVED3[1];
  volatile unsigned long FFA0R;
  unsigned long RESERVED4[1];
  volatile unsigned long FA0R;
  unsigned long RESERVED5[8];
  CAN_FilterRegister_TypeDef sFilterRegister[14];
} CAN_TypeDef;

/*------------------------ DMA Controller ------------------------------------*/
typedef struct
{
  volatile unsigned long CCR;
  volatile unsigned long CNDTR;
  volatile unsigned long CPAR;
  volatile unsigned long CMAR;
} DMA_Channel_TypeDef;

typedef struct
{
  volatile unsigned long ISR;
  volatile unsigned long IFCR;
} DMA_TypeDef;

/*------------------------ External Interrupt/Event Controller ---------------*/
typedef struct
{
  volatile unsigned long IMR;
  volatile unsigned long EMR;
  volatile unsigned long RTSR;
  volatile unsigned long FTSR;
  volatile unsigned long SWIER;
  volatile unsigned long PR;
} EXTI_TypeDef;

/*------------------------ FLASH and Option Bytes Registers ------------------*/
typedef struct
{
  volatile unsigned long ACR;
  volatile unsigned long KEYR;
  volatile unsigned long OPTKEYR;
  volatile unsigned long SR;
  volatile unsigned long CR;
  volatile unsigned long AR;
  volatile unsigned long RESERVED;
  volatile unsigned long OBR;
  volatile unsigned long WRPR;
} FLASH_TypeDef;

typedef struct
{
  volatile unsigned short RDP;
  volatile unsigned short USER;
  volatile unsigned short Data0;
  volatile unsigned short Data1;
  volatile unsigned short WRP0;
  volatile unsigned short WRP1;
  volatile unsigned short WRP2;
  volatile unsigned short WRP3;
} OB_TypeDef;

/*------------------------ General Purpose and Alternate Function IO ---------*/
typedef struct
{
  volatile unsigned long CRL;
  volatile unsigned long CRH;
  volatile unsigned long IDR;
  volatile unsigned long ODR;
  volatile unsigned long BSRR;
  volatile unsigned long BRR;
  volatile unsigned long LCKR;
} GPIO_TypeDef;

typedef struct
{
  volatile unsigned long EVCR;
  volatile unsigned long MAPR;
  volatile unsigned long EXTICR[4];
} AFIO_TypeDef;

/*------------------------ Inter-integrated Circuit Interface ----------------*/
typedef struct
{
  volatile unsigned short CR1;
  unsigned short RESERVED0;
  volatile unsigned short CR2;
  unsigned short RESERVED1;
  volatile unsigned short OAR1;
  unsigned short RESERVED2;
  volatile unsigned short OAR2;
  unsigned short RESERVED3;
  volatile unsigned short DR;
  unsigned short RESERVED4;
  volatile unsigned short SR1;
  unsigned short RESERVED5;
  volatile unsigned short SR2;
  unsigned short RESERVED6;
  volatile unsigned short CCR;
  unsigned short RESERVED7;
  volatile unsigned short TRISE;
  unsigned short RESERVED8;
} I2C_TypeDef;

/*------------------------ Independent WATCHDOG ------------------------------*/
typedef struct
{
  volatile unsigned long KR;
  volatile unsigned long PR;
  volatile unsigned long RLR;
  volatile unsigned long SR;
} IWDG_TypeDef;

/*------------------------ Nested Vectored Interrupt Controller --------------*/
typedef struct
{
  volatile unsigned long Enable[2];
  unsigned long RESERVED0[30];
  volatile unsigned long Disable[2];
  unsigned long RSERVED1[30];
  volatile unsigned long Set[2];
  unsigned long RESERVED2[30];
  volatile unsigned long Clear[2];
  unsigned long RESERVED3[30];
  volatile unsigned long Active[2];
  unsigned long RESERVED4[62];
  volatile unsigned long Priority[11];
} NVIC_TypeDef;

typedef struct
{
  volatile unsigned long CPUID;
  volatile unsigned long IRQControlState;
  volatile unsigned long ExceptionTableOffset;
  volatile unsigned long AIRC;
  volatile unsigned long SysCtrl;
  volatile unsigned long ConfigCtrl;
  volatile unsigned long SystemPriority[3];
  volatile unsigned long SysHandlerCtrl;
  volatile unsigned long ConfigFaultStatus;
  volatile unsigned long HardFaultStatus;
  volatile unsigned long DebugFaultStatus;
  volatile unsigned long MemoryManageFaultAddr;
  volatile unsigned long BusFaultAddr;
} SCB_TypeDef;

/*------------------------ Power Control -------------------------------------*/
typedef struct
{
  volatile unsigned long CR;
  volatile unsigned long CSR;
} PWR_TypeDef;

/*------------------------ Reset and Clock Control ---------------------------*/
typedef struct
{
  volatile unsigned long CR;
  volatile unsigned long CFGR;
  volatile unsigned long CIR;
  volatile unsigned long APB2RSTR;
  volatile unsigned long APB1RSTR;
  volatile unsigned long AHBENR;
  volatile unsigned long APB2ENR;
  volatile unsigned long APB1ENR;
  volatile unsigned long BDCR;
  volatile unsigned long CSR;
} RCC_TypeDef;

/*------------------------ Real-Time Clock -----------------------------------*/
typedef struct
{
  volatile unsigned short CRH;
  unsigned short RESERVED0;
  volatile unsigned short CRL;
  unsigned short RESERVED1;
  volatile unsigned short PRLH;
  unsigned short RESERVED2;
  volatile unsigned short PRLL;
  unsigned short RESERVED3;
  volatile unsigned short DIVH;
  unsigned short RESERVED4;
  volatile unsigned short DIVL;
  unsigned short RESERVED5;
  volatile unsigned short CNTH;
  unsigned short RESERVED6;
  volatile unsigned short CNTL;
  unsigned short RESERVED7;
  volatile unsigned short ALRH;
  unsigned short RESERVED8;
  volatile unsigned short ALRL;
  unsigned short RESERVED9;
} RTC_TypeDef;

/*------------------------ Serial Peripheral Interface -----------------------*/
typedef struct
{
  volatile unsigned short CR1;
  unsigned short RESERVED0;
  volatile unsigned short CR2;
  unsigned short RESERVED1;
  volatile unsigned short SR;
  unsigned short  RESERVED2;
  volatile unsigned short DR;
  unsigned short  RESERVED3;
  volatile unsigned short CRCPR;
  unsigned short RESERVED4;
  volatile unsigned short RXCRCR;
  unsigned short  RESERVED5;
  volatile unsigned short TXCRCR;
  unsigned short  RESERVED6;
} SPI_TypeDef;

/*------------------------ SystemTick ----------------------------------------*/
typedef struct
{
  volatile unsigned long CTRL;
  volatile unsigned long LOAD;
  volatile unsigned long VAL;
  volatile unsigned long const CALIB;
} SysTick_TypeDef;

/*------------------------ Advanced Control Timer ----------------------------*/
typedef struct
{
  volatile unsigned short CR1;
  unsigned short RESERVED0;
  volatile unsigned short CR2;
  unsigned short RESERVED1;
  volatile unsigned short SMCR;
  unsigned short RESERVED2;
  volatile unsigned short DIER;
  unsigned short RESERVED3;
  volatile unsigned short SR;
  unsigned short RESERVED4;
  volatile unsigned short EGR;
  unsigned short RESERVED5;
  volatile unsigned short CCMR1;
  unsigned short RESERVED6;
  volatile unsigned short CCMR2;
  unsigned short RESERVED7;
  volatile unsigned short CCER;
  unsigned short RESERVED8;
  volatile unsigned short CNT;
  unsigned short RESERVED9;
  volatile unsigned short PSC;
  unsigned short RESERVED10;
  volatile unsigned short ARR;
  unsigned short RESERVED11;
  volatile unsigned short RCR;
  unsigned short RESERVED12;
  volatile unsigned short CCR1;
  unsigned short RESERVED13;
  volatile unsigned short CCR2;
  unsigned short RESERVED14;
  volatile unsigned short CCR3;
  unsigned short RESERVED15;
  volatile unsigned short CCR4;
  unsigned short RESERVED16;
  volatile unsigned short BDTR;
  unsigned short RESERVED17;
  volatile unsigned short DCR;
  unsigned short RESERVED18;
  volatile unsigned short DMAR;
  unsigned short RESERVED19;
} TIM1_TypeDef;

/*------------------------ General Purpose Timer -----------------------------*/
typedef struct
{
  volatile unsigned short CR1;
  unsigned short RESERVED0;
  volatile unsigned short CR2;
  unsigned short RESERVED1;
  volatile unsigned short SMCR;
  unsigned short RESERVED2;
  volatile unsigned short DIER;
  unsigned short RESERVED3;
  volatile unsigned short SR;
  unsigned short RESERVED4;
  volatile unsigned short EGR;
  unsigned short RESERVED5;
  volatile unsigned short CCMR1;
  unsigned short RESERVED6;
  volatile unsigned short CCMR2;
  unsigned short RESERVED7;
  volatile unsigned short CCER;
  unsigned short RESERVED8;
  volatile unsigned short CNT;
  unsigned short RESERVED9;
  volatile unsigned short PSC;
  unsigned short RESERVED10;
  volatile unsigned short ARR;
  unsigned short RESERVED11[3];
  volatile unsigned short CCR1;
  unsigned short RESERVED12;
  volatile unsigned short CCR2;
  unsigned short RESERVED13;
  volatile unsigned short CCR3;
  unsigned short RESERVED14;
  volatile unsigned short CCR4;
  unsigned short RESERVED15[3];
  volatile unsigned short DCR;
  unsigned short RESERVED16;
  volatile unsigned short DMAR;
  unsigned short RESERVED17;
} TIM_TypeDef;

/*----------------- Universal Synchronous Asynchronous Receiver Transmitter --*/
typedef struct
{
  volatile unsigned short SR;
  unsigned short RESERVED0;
  volatile unsigned short DR;
  unsigned short RESERVED1;
  volatile unsigned short BRR;
  unsigned short RESERVED2;
  volatile unsigned short CR1;
  unsigned short RESERVED3;
  volatile unsigned short CR2;
  unsigned short RESERVED4;
  volatile unsigned short CR3;
  unsigned short RESERVED5;
  volatile unsigned short GTPR;
  unsigned short RESERVED6;
} USART_TypeDef;

/*------------------------ Window WATCHDOG -----------------------------------*/
typedef struct
{
  volatile unsigned long CR;
  volatile unsigned long CFR;
  volatile unsigned long SR;
} WWDG_TypeDef;

extern TIM_TypeDef             _TIM2;
#define TIM2 (&_TIM2)

extern TIM_TypeDef             _TIM3;
#define TIM3 (&_TIM3)

extern TIM_TypeDef             _TIM4;
#define TIM4 (&_TIM4)

extern RTC_TypeDef             _RTC;
#define RTC (&_RTC)

extern WWDG_TypeDef            _WWDG;
#define WWDG (&_WWDG)

extern IWDG_TypeDef            _IWDG;
#define IWDG (&_IWDG)

extern SPI_TypeDef             _SPI2;
#define SPI2 (&_SPI2)

extern USART_TypeDef           _USART2;
#define USART2 (&_USART2)

extern USART_TypeDef           _USART3;
#define USART3 (&_USART3)

extern I2C_TypeDef             _I2C1;
#define I2C1 (&_I2C1)

extern I2C_TypeDef             _I2C2;
#define I2C2 (&_I2C2)

extern CAN_TypeDef             _CAN;
#define CAN (&_CAN)

extern BKP_TypeDef             _BKP;
#define BKP (&_BKP)

extern PWR_TypeDef             _PWR;
#define PWR (&_PWR)

extern AFIO_TypeDef            _AFIO;
#define AFIO (&_AFIO)

extern EXTI_TypeDef            _EXTI;
#define EXTI (&_EXTI)

extern GPIO_TypeDef            _GPIOA;
#define GPIOA (&_GPIOA)

extern GPIO_TypeDef            _GPIOB;
#define GPIOB (&_GPIOB)

extern GPIO_TypeDef            _GPIOC;
#define GPIOC (&_GPIOC)

extern GPIO_TypeDef            _GPIOD;
#define GPIOD (&_GPIOD)

extern GPIO_TypeDef            _GPIOE;
#define GPIOE (&_GPIOE)

extern ADC_TypeDef             _ADC1;
#define ADC1 (&_ADC1)

extern ADC_TypeDef             _ADC2;
#define ADC2 (&_ADC2)

extern TIM1_TypeDef            _TIM1;
#define TIM1 (&_TIM1)

extern SPI_TypeDef             _SPI1;
#define SPI1 (&_SPI1)

extern USART_TypeDef           _USART1;
#define USART1 (&_USART1)

extern DMA_TypeDef             _DMA;
#define DMA (&_DMA)

extern DMA_Channel_TypeDef     _DMA_Channel1;
#define DMA_Channel1 (&_DMA_Channel1)

extern DMA_Channel_TypeDef     _DMA_Channel2;
#define DMA_Channel2 (&_DMA_Channel2)

extern DMA_Channel_TypeDef     _DMA_Channel3;
#define DMA_Channel3 (&_DMA_Channel3)

extern DMA_Channel_TypeDef     _DMA_Channel4;
#define DMA_Channel4 (&_DMA_Channel4)

extern DMA_Channel_TypeDef     _DMA_Channel5;
#define DMA_Channel5 (&_DMA_Channel5)

extern DMA_Channel_TypeDef     _DMA_Channel6;
#define DMA_Channel6 (&_DMA_Channel6)

extern DMA_Channel_TypeDef     _DMA_Channel7;
#define DMA_Channel7 (&_DMA_Channel7)

extern FLASH_TypeDef            _FLASH;
#define FLASH (&_FLASH)

extern OB_TypeDef               _OB;  
#define OB (&_OB)

extern RCC_TypeDef             _RCC;
#define RCC (&_RCC)

extern SysTick_TypeDef         _SysTick;
#define SysTick (&_SysTick)

extern NVIC_TypeDef            _NVIC;
#define NVIC (&_NVIC)

extern SCB_TypeDef             _SCB;
#define SCB (&_SCB)
