//
// Ride7 Project Example for J-Link/GDB/DDD Debugging "Tutorial" (...)
// ASkr
//


#define _RCC

#include "stm32f10x_lib.h"
#include "asSerial.h"


void InitPorts(void);
void RCC_Configuration(void);
void NVIC_Configuration(void);
int asInitPorts(void);


ErrorStatus HSEStartUpStatus;


//************************************************************************************************
//*** RCC_Configuration
//***
//***
//************************************************************************************************
void RCC_Configuration(void)
{
	// just a quick hack...
	// do not use this for real programs ;)
  RCC_DeInit();
  RCC_HSEConfig(RCC_HSE_ON);
  HSEStartUpStatus = RCC_WaitForHSEStartUp();

  if(HSEStartUpStatus == SUCCESS)
  {
    FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);
    FLASH_SetLatency(FLASH_Latency_2);
    RCC_HCLKConfig(RCC_SYSCLK_Div1); 
    RCC_PCLK2Config(RCC_HCLK_Div1); 
    RCC_PCLK1Config(RCC_HCLK_Div2);
    RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);
    RCC_PLLCmd(ENABLE);
    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET)
    {;}
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);
    while(RCC_GetSYSCLKSource() != 0x08)
    {;}
  }
	else
		// lock
		while(1)
		{;}
}



//************************************************************************************************
//*** NVIC_Configuration
//***
//***
//************************************************************************************************
void NVIC_Configuration(void)
{
#ifdef  VECT_TAB_RAM  
  /* Set the Vector Table base location at 0x20000000 */ 
  NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0); 
#else  /* VECT_TAB_FLASH  */
  /* Set the Vector Table base location at 0x08000000 */ 
  NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);   
#endif
}



//************************************************************************************************
//*** asInitPorts
//***
//***
//************************************************************************************************
int asInitPorts()
{
  GPIO_InitTypeDef sPit;

  GPIO_StructInit(&sPit);

  GPIO_PinRemapConfig(GPIO_FullRemap_TIM3,DISABLE);

  // PORT C
  sPit.GPIO_Pin=GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9;
  sPit.GPIO_Speed=GPIO_Speed_50MHz;
  sPit.GPIO_Mode=GPIO_Mode_Out_PP;
  GPIO_Init(GPIOC, &sPit);
  GPIO_SetBits(GPIOC,GPIO_Pin_6 | GPIO_Pin_8);
  GPIO_ResetBits(GPIOC,GPIO_Pin_7 | GPIO_Pin_9);

  // PORT B
  sPit.GPIO_Pin=GPIO_Pin_5;
  sPit.GPIO_Speed=GPIO_Speed_50MHz;
  sPit.GPIO_Mode=GPIO_Mode_Out_PP;
  GPIO_Init(GPIOB, &sPit);
  GPIO_SetBits(GPIOB,GPIO_Pin_5);

}





//************************************************************************************************
//*** main
//***
//***
//************************************************************************************************
int main(int argc, char **argv) // well, everyone likes arguments, right? ;-)
{
  char data0[60];
  char data1[60];

  RCC_Configuration();
  NVIC_Configuration();
  asInitPorts();
  asSerialInit();

	
  for(;;)
  {
		// create a really fast pulse (PB5)
    GPIO_SetBits(GPIOB,GPIO_Pin_5);
    GPIO_ResetBits(GPIOB,GPIO_Pin_5);

		// put out some text on UART1 (19200, 8N1)
    asSerialSendString("\r\nEnter some text: ");
		
		// receive a string (blocking)
    asSerialReceiveString(&data0,58);
		
    asSerialSendString("\r\n>");
    asSerialSendString(&data0);
  }

    
}// END main

